@@ -1,19 +1,19 @@
 import linecache
 
+from typing import Any, AsyncGenerator, Optional, Tuple
+
 from .state import Machine
 
 
-# __________________________________________________________________||
 class Nextline:
     """Nextline allows line-by-line execution of concurrent Python scripts
 
-    Nextline supports concurrency with threading and asyncio. It uses
-    multiple instances of Pdb, one for each thread and async task.
+    Nextline supports concurrency with threading and asyncio. It uses multiple
+    instances of Pdb, one for each thread and async task.
 
     Note
     ----
-    The running asyncio event loop must exists when Nextline is
-    instantiated.
+    The running asyncio event loop must exists when Nextline is instantiated.
 
     Parameters
     ----------
@@ -28,35 +28,30 @@ class Nextline:
 
     def __repr__(self):
         # e.g., "<Nextline 'running'>"
-        return f"<{self.__class__.__name__} {self.global_state!r}>"
+        return f"<{self.__class__.__name__} {self.state!r}>"
 
-    def run(self):
-        """execute the script"""
+    def run(self) -> None:
+        """Execute the script"""
         self.machine.run()
 
-    async def finish(self):
-        """wait until the script execution exits
-
-        wait for the script execution in another thread to exit and
-        join the thread.
-
-        """
+    async def finish(self) -> None:
+        """Wait until the script execution exits"""
         await self.machine.finish()
 
-    def exception(self):
-        """uncaught exception from the last run"""
+    def exception(self) -> Optional[Exception]:
+        """Uncaught exception from the last run"""
         return self.machine.exception()
 
-    def result(self):
-        """return value of the last run. always None"""
-        self.machine.result()
+    def result(self) -> Any:
+        """Return value of the last run. always None"""
+        return self.machine.result()
 
-    def reset(self, statement=None):
-        """prepare for run"""
+    def reset(self, statement=None) -> None:
+        """Prepare for the next run"""
         self.machine.reset(statement=statement)
 
-    async def close(self):
-        """end gracefully"""
+    async def close(self) -> None:
+        """End gracefully"""
         await self.machine.close()
 
     @property
@@ -65,11 +60,15 @@ class Nextline:
         return self.registry.get("statement")
 
     @property
-    def global_state(self) -> str:
-        """state, e.g., "initialized", "running", "exited", "finished", "closed" """
+    def state(self) -> str:
+        """The current condition of the script execution.
+
+        The possible values are "initialized", "running", "exited", "finished",
+        "closed"
+        """
         return self.machine.state_name
 
-    async def subscribe_global_state(self):
+    async def subscribe_state(self) -> AsyncGenerator[str, None]:
         # wish to be able to write with "yield from" but not possible
         # https://stackoverflow.com/a/59079548/7309855
         agen = self.registry.subscribe("state_name")
@@ -86,18 +85,18 @@ class Nextline:
         async for y in agen:
             yield y
 
-    async def subscribe_thread_asynctask_ids(self):
-        agen = self.registry.subscribe("thread_task_ids")
+    async def subscribe_trace_ids(self) -> AsyncGenerator[Tuple[int], None]:
+        agen = self.registry.subscribe("trace_ids")
         async for y in agen:
             yield y
 
-    async def subscribe_thread_asynctask_state(self, thread_asynctask_id):
-        agen = self.registry.subscribe(thread_asynctask_id)
+    async def subscribe_trace_state(self, trace_id: int):
+        agen = self.registry.subscribe(trace_id)
         async for y in agen:
             yield y
 
-    def send_pdb_command(self, thread_asynctask_id, command):
-        self.machine.send_pdb_command(thread_asynctask_id, command)
+    def send_pdb_command(self, trace_id: int, command: str):
+        self.machine.send_pdb_command(trace_id, command)
 
     def get_source(self, file_name=None):
         if not file_name or file_name == self.registry.get("script_file_name"):
@@ -113,6 +112,3 @@ class Nextline:
         if 1 <= line_no <= len(lines):
             return lines[line_no - 1]
         return ""
-
-
-# __________________________________________________________________||
