@@ -1,7 +1,9 @@
+from __future__ import annotations
+
 import asyncio
 from threading import Thread
 import itertools
-from typing import Union
+from typing import Optional, Any
 
 from .trace import Trace
 from .registry import PdbCIRegistry
@@ -48,20 +50,21 @@ class Machine:
 
     def __init__(self, statement: str, run_no_start_from=1):
         self.registry = Registry()
-        self.registry.open_register("statement")
-        self.registry.open_register("state_name")
-        self.registry.open_register("script_file_name")
-        self.registry.open_register("run_no")
-        self.registry.open_register("run_no_count")
-        self.registry.open_register_list("thread_task_ids")
+        # self.registry.open_register("statement")
+        # self.registry.open_register("state_name")
+        # self.registry.open_register("script_file_name")
+        # self.registry.open_register("run_no")
+        # self.registry.open_register("run_no_count")
+        # self.registry.open_register("thread_task_ids")
 
         self.registry.register("statement", statement)
         self.registry.register("script_file_name", SCRIPT_FILE_NAME)
         self.registry.register(
             "run_no_count", itertools.count(run_no_start_from).__next__
         )
+        self.registry.register("thread_task_ids", ())
 
-        self._state = Initialized(self.registry)
+        self._state: State = Initialized(self.registry)
 
         self._lock_finish = asyncio.Condition()
         self._lock_close = asyncio.Condition()
@@ -78,12 +81,12 @@ class Machine:
         except BaseException:
             return "unknown"
 
-    def run(self):
+    def run(self) -> None:
         """Enter the running state"""
         self._state = self._state.run()
         self._task_exited = asyncio.create_task(self._exited())
 
-    async def _exited(self):
+    async def _exited(self) -> None:
         """Enter the exited state
 
         This method is scheduled as a task in run().
@@ -94,32 +97,32 @@ class Machine:
 
         self._state = await self._state.exited()
 
-    def send_pdb_command(self, thread_asynctask_id, command):
+    def send_pdb_command(self, thread_asynctask_id, command) -> None:
         self._state.send_pdb_command(thread_asynctask_id, command)
 
-    async def finish(self):
+    async def finish(self) -> None:
         """Enter the finished state"""
         await self._task_exited
         async with self._lock_finish:
             self._state = await self._state.finish()
 
-    def exception(self):
+    def exception(self) -> Optional[Exception]:
         return self._state.exception()
 
-    def result(self):
-        self._state.result()
+    def result(self) -> Any:
+        return self._state.result()
 
-    def reset(self, statement: Union[str, None] = None):
+    def reset(self, statement: Optional[str] = None) -> None:
         """Enter the initialized state"""
         if statement:
             self.registry.register("statement", statement)
         self._state = self._state.reset()
 
-    async def close(self):
+    async def close(self) -> None:
         """Enter the closed state"""
         async with self._lock_close:
             self._state = self._state.close()
-            await self.registry.close()
+            await to_thread(self.registry.close)
 
 
 # __________________________________________________________________||
@@ -151,6 +154,8 @@ class ObsoleteMixin:
 class State(ObsoleteMixin):
     """The base state class in the Nextline state machine"""
 
+    name = "state"
+
     def __repr__(self):
         # e.g., "<Initialized 'initialized'>"
         items = [self.__class__.__name__, repr(self.name)]
@@ -158,33 +163,33 @@ class State(ObsoleteMixin):
             items.append("obsolete")
         return f'<{" ".join(items)}>'
 
-    def run(self):
+    def run(self) -> State:
         self.assert_not_obsolete()
         raise StateMethodError(f"Irrelevant operation on the state: {self!r}")
 
-    async def exited(self):
+    async def exited(self) -> State:
         self.assert_not_obsolete()
         raise StateMethodError(f"Irrelevant operation on the state: {self!r}")
 
-    async def finish(self):
+    async def finish(self) -> State:
         self.assert_not_obsolete()
         raise StateMethodError(f"Irrelevant operation on the state: {self!r}")
 
-    def reset(self):
+    def reset(self) -> State:
         self.assert_not_obsolete()
         raise StateMethodError(f"Irrelevant operation on the state: {self!r}")
 
-    def close(self):
+    def close(self) -> State:
         self.assert_not_obsolete()
         raise StateMethodError(f"Irrelevant operation on the state: {self!r}")
 
-    def send_pdb_command(self, *_, **__):
+    def send_pdb_command(self, *_, **__) -> None:
         raise StateMethodError(f"Irrelevant operation on the state: {self!r}")
 
-    def exception(self):
+    def exception(self) -> Optional[Exception]:
         raise StateMethodError(f"Irrelevant operation on the state: {self!r}")
 
-    def result(self):
+    def result(self) -> Any:
         raise StateMethodError(f"Irrelevant operation on the state: {self!r}")
 
 
@@ -253,31 +258,31 @@ class Running(State):
 
         self.loop = asyncio.get_running_loop()
 
-        self._thread = Thread(
-            target=self._run, args=(statement, trace), daemon=True
-        )
+        def run():
+            code = statement
+            if isinstance(code, str):
+                try:
+                    code = compile(code, SCRIPT_FILE_NAME, "exec")
+                except BaseException as e:
+                    self._done(None, e)
+                    return
+            func = script.compose(code)
+            result = None
+            exception = None
+            try:
+                result = call_with_trace(func, trace)
+            except BaseException as e:
+                exception = e
+            self._done(result, exception)
+
+        self._thread = Thread(target=run, daemon=True)
         self._thread.start()
 
-    def _run(self, code, trace):
-
-        if isinstance(code, str):
-            try:
-                code = compile(code, SCRIPT_FILE_NAME, "exec")
-            except BaseException as e:
-                self._done(None, e)
-                return
-
-        func = script.compose(code)
-
-        call_with_trace(func, trace, self._done)
-
     def _done(self, result=None, exception=None):
-        # callback function, to be called from another thread at the
-        # end of _run()
 
         if self.loop.is_closed():
-            # The exit is not being waited in the main thread, for
-            # example, neither exited() of finish() is called.
+            # The exit is not being waited in the main thread, for example,
+            # exited() is not called.
             return
 
         self._exited = Exited(
@@ -287,8 +292,6 @@ class Running(State):
             exception=exception,
         )
 
-        self.obsolete()
-
         try:
             self._event.set()
         except RuntimeError:
@@ -298,6 +301,7 @@ class Running(State):
     async def exited(self):
         """return the exited state after the script exits."""
         await self._event.wait()
+        self.obsolete()
         return self._exited
 
     def send_pdb_command(self, thread_asynctask_id, command):
