@@ -4,19 +4,20 @@ from weakref import WeakKeyDictionary
 
 import fnmatch
 
-from typing import Any, Set, Dict, Optional, Union, Callable, TYPE_CHECKING
+from typing import Any, Set, Optional, Union, Callable, TYPE_CHECKING
 from types import FrameType
 
-from .pdb.proxy import PdbInterfaceFactory, PdbInterface
-from .registry import PdbCIRegistry
+from .pdb.proxy import PdbInterfaceFactory
 from .utils import current_task_or_thread
 
-from .types import TraceFunc
-
 if TYPE_CHECKING:
+    from .types import TraceFunc
     from .utils import Registry
+    from .pdb.proxy import PdbInterface
+    from .registry import PdbCIRegistry
 
-MODULES_TO_SKIP = [
+
+MODULES_TO_SKIP = {
     "threading",
     "queue",
     "importlib",
@@ -42,7 +43,7 @@ MODULES_TO_SKIP = [
     "nextline.utils.*",
     "nextline.queuedist",
     "nextlinegraphql.schema.bindables",
-]
+}
 
 
 def Trace(
@@ -104,7 +105,7 @@ def TraceSkipModule(trace: TraceFunc, skip: Set[str]) -> TraceFunc:
     def ret(frame: FrameType, event, arg) -> Optional[TraceFunc]:
         module_name = frame.f_globals.get("__name__")
         if _is_matched_to_any(module_name, skip):
-            return
+            return None
         return trace(frame, event, arg)
 
     return ret
@@ -114,7 +115,7 @@ def TraceSkipLambda(trace: TraceFunc) -> TraceFunc:
     def ret(frame: FrameType, event, arg) -> Optional[TraceFunc]:
         func_name = frame.f_code.co_name
         if func_name == "<lambda>":
-            return
+            return None
         return trace(frame, event, arg)
 
     return ret
@@ -125,22 +126,35 @@ def TraceAddFirstModule(
 ) -> TraceFunc:
     first = True
 
-    def ret(frame: FrameType, event, arg) -> Optional[TraceFunc]:
-        nonlocal first
-        if first:
-            module_name = frame.f_globals.get("__name__")
-            modules_to_trace.add(module_name)
-            first = False
+    def global_trace(frame: FrameType, event, arg) -> Optional[TraceFunc]:
+        if not first:
+            return trace(frame, event, arg)
 
-        return trace(frame, event, arg)
+        def create_local_trace():
+            next_trace: Union[TraceFunc, None] = trace
 
-    return ret
+            def local_trace(frame, event, arg):
+                nonlocal first, next_trace
+
+                if module_name := frame.f_globals.get("__name__"):
+                    first = False
+                    modules_to_trace.add(module_name)
+                    return trace(frame, event, arg)
+
+                next_trace = next_trace(frame, event, arg)
+                return local_trace
+
+            return local_trace
+
+        return create_local_trace()(frame, event, arg)
+
+    return global_trace
 
 
 def TraceDispatchThreadOrTask(factory: Callable[[], TraceFunc]) -> TraceFunc:
     """Create a trace that creates a new trace for each thread or asyncio task"""
 
-    map: Dict[Any, TraceFunc] = WeakKeyDictionary()
+    map: WeakKeyDictionary[Any, TraceFunc] = WeakKeyDictionary()
 
     def ret(frame, event, arg) -> Optional[TraceFunc]:
         key = current_task_or_thread()
@@ -164,7 +178,7 @@ def TraceSelectFirstModule(
         if first:
             module_name = frame.f_globals.get("__name__")
             if not _is_matched_to_any(module_name, modules_to_trace):
-                return
+                return None
             first = False
         return trace(frame, event, arg)
 
