@@ -1,5 +1,6 @@
 import asyncio
 from pathlib import Path
+from typing import Set
 
 import pytest
 
@@ -8,7 +9,7 @@ from nextline import Nextline
 ##__________________________________________________________________||
 statement = """
 import time
-time.sleep(0.1)
+time.sleep(0.001)
 
 def f():
     for _ in range(10):
@@ -37,22 +38,23 @@ def monkey_patch_syspath(monkeypatch):
 
 
 ##__________________________________________________________________||
-async def monitor_global_state(nextline):
-    async for s in nextline.subscribe_global_state():
-        print("monitor_global_state()", s)
+async def monitor_state(nextline: Nextline):
+    async for s in nextline.subscribe_state():
+        # print("monitor_state()", s)
+        pass
 
 
-async def control_execution(nextline):
-    prev_ids = set()
-    tasks = set()
+async def control_execution(nextline: Nextline):
+    prev_ids: Set[int] = set()
+    tasks: Set[asyncio.Task] = set()
 
     # The lines of code between ============= can be rewritten with
     # one line of code as
-    #   async for ids in nextline.subscribe_thread_asynctask_ids():
+    #   async for ids in nextline.subscribe_trace_ids():
     # if exceptions occurred in tasks don't need to be re-raised.
 
     # ==================================================
-    subscription = nextline.subscribe_thread_asynctask_ids()
+    subscription = nextline.subscribe_trace_ids()
     task_anext = asyncio.create_task(subscription.__anext__())
     while True:
         aws = {task_anext, *tasks}
@@ -66,28 +68,28 @@ async def control_execution(nextline):
         if not task_anext.done():
             continue
         try:
-            ids = task_anext.result()
+            ids_ = task_anext.result()
         except StopAsyncIteration:
             break
         task_anext = asyncio.create_task(subscription.__anext__())
         # ===============================================
 
-        ids = set(ids)
+        ids = set(ids_)
 
         new_ids = ids - prev_ids
 
         for id_ in new_ids:
-            task = asyncio.create_task(control_thread_task(nextline, id_))
+            task = asyncio.create_task(control_trace(nextline, id_))
             tasks.add(task)
 
         prev_ids = ids
 
 
-async def control_thread_task(nextline, thread_task_id):
+async def control_trace(nextline: Nextline, trace_id):
     to_step = ["script_threading.run()", "script_asyncio.run()"]
-    print(f"control_thread_task({thread_task_id})")
+    # print(f"control_trace({trace_id})")
     file_name = ""
-    async for s in nextline.subscribe_thread_asynctask_state(thread_task_id):
+    async for s in nextline.subscribe_trace_state(trace_id):
         # print(s)
         if not file_name == s["file_name"]:
             file_name = s["file_name"]
@@ -101,10 +103,10 @@ async def control_thread_task(nextline, thread_task_id):
                 if line in to_step:
                     # print(line)
                     command = "step"
-            nextline.send_pdb_command(thread_task_id, command)
+            nextline.send_pdb_command(trace_id, command)
 
 
-async def run(nextline):
+async def run(nextline: Nextline):
     await asyncio.sleep(0.1)
     nextline.run()
     await nextline.finish()
@@ -119,20 +121,18 @@ async def test_run():
 
     nextline = Nextline(statement)
 
-    assert nextline.global_state == "initialized"
+    assert nextline.state == "initialized"
 
-    task_monitor_global_state = asyncio.create_task(
-        monitor_global_state(nextline)
-    )
+    task_monitor_state = asyncio.create_task(monitor_state(nextline))
 
     task_control_execution = asyncio.create_task(control_execution(nextline))
 
     task_run = asyncio.create_task(run(nextline))
 
-    aws = [task_run, task_monitor_global_state, task_control_execution]
+    aws = [task_run, task_monitor_state, task_control_execution]
     await asyncio.gather(*aws)
 
-    assert nextline.global_state == "closed"
+    assert nextline.state == "closed"
 
 
 ##__________________________________________________________________||
