@@ -1,6 +1,6 @@
 from . import module_b
 
 
-def func():
-    module_b.func()
+def func_a():
+    module_b.func_b()
     return
